import sys

print sys.path

from basicAnalysis import replicatesGraph, hclust, MDS, printClustering, unpckGS

import orange

import stats
import mMisc as m

defver = 2

addressS = "http://purple.bioch.bcm.tmc.edu/~anup/index.php?token=gFl0g7y5lefcnY6mqAdLd&"

def getBasicData2():

    import obiDicty
    dbc = obiDicty.DatabaseConnection(addressS)
    etso = dbc.getData(platform='minichip', separate=["techReplicate", "sample"])
    #to get A5 sample - A5 has two different biological replicates
    etsoA5 = dbc.getData(platform='minichip', sample="abcA5-", separate=["techReplicate", "replicate", "sample"])

    ets = []

    for et in etso + etsoA5:
        new = orngDicty.averageAttributes(et)
        new.annot = et.annot

        if len(new.domain.attributes) != 4:
            print "skipped", new.annot
            continue # skip those with more attributes

        ets.append(new)

    return ets

def getBasicDataPka(samples):
    
    import obiDicty

    dbc = obiDicty.DatabaseConnection(addressS, buffer=obiDicty.Buffer1("../bufsql1.sq3", compress=True))

    obiDicty.verbose = 1

    type = "norms"

    ids = dbc.search(type, sample = samples, growthCond="hl-5", treatment="none")

    read = dbc.dictionarize(ids, dbc.annotations, type, ids)
    gannots = dbc.groupAnnotations(read, join=["time"])

    gannots = sorted(gannots, key=lambda x: list(x[1]["sample"])[0])

    print len(ids)

    taketimes = [ "time=%d" % a for a in range(0,25,2)]

    def rightTimePoints(x):
        return set(m.nth(x[0],0)) & set(taketimes)

    gannots = filter(lambda x: len(rightTimePoints(x)) > 8, gannots)

    print len(gannots)

    """
    for a in gannots:
        print list(a[1]["sample"])[0], a
        print len(a[0])
    """

    for a in gannots:
        print list(a[1]["sample"])[0], [ b[0] for b in a[0] ]

    dsd =  m.flatten([ [ b[1] for b in a[0] ] for a in gannots ])
    print len(dsd)

    chipd = dbc.dictionarize(dsd, dbc.chipNs, dsd)

    #create example tables grouped according to user's wishes
    ets = dbc.exampleTables(chipd, gannots, spotmap=dbc.spotMap())
    chipd = None

    averageEts = []
    for et in ets:
        av = obiDicty.averageAttributes(et)
        av.annot = et.annot
        averageEts.append(av)
    ets = None

    #fill unknown atributes in case of 
    fillBlanks = []
    for et in averageEts:
        examples = []
        ddbs = []
        names = taketimes
        print names
        for ex in et:
            nex = []
            for doc in taketimes:
                try:
                    nex.append(obiDicty.floatOrUnknown(ex[doc]))
                except:
                    nex.append("?")
            ddbs.append(ex["DDB"])
            examples.append(nex)
        net = obiDicty.createExampleTable(names, zip(*examples), ddbs) 
        net.annot = et.annot
        fillBlanks.append(net)

    return fillBlanks

def hash(s):
    """
    My hashing function.
    """
    import hashlib
    return hashlib.sha256(s).hexdigest()

def getBasicDataPck():
    ver = defver
    if ver == 2:
        # DDB's are in the same order for all files
        # each file contains one technical replicate
        # each mutant, except a5, has one replicate
        return m.autoPck("bd_obiDicty.1.pck", "3", getBasicData2)
    elif ver[:5] == "pkar " or ver[:5] == 'pka2 ':
        ver = 'pkar ' + ver[5:]
        return m.autoPck("bd_obiDicty.1." + hash(ver) + ".pck", "3", getBasicDataPka, eval(ver[5:]))
        #return getBasicDataPka(eval(ver[5:]))

def attributeValues(data, attribute, avfn=str):
    return [ avfn(e[attribute]) for e in data ]

def exampleValues(data, e, avfn=str, clfn=str):
    return [ avfn(e[at]) for at  in range(len(data.domain.attributes)) ] + \
        [ clfn(e[-1])]
        
def samplesDict(ets, onlyabc=False):

    mutants = {}

    def isabc(m):
        return m.lower()[:3] == "abc"

    for i,et in enumerate(ets):
        mutant = sorted(et.annot["sample"])[0]
        if not onlyabc or (onlyabc and isabc(mutant)):
            num = len([ 1 for a in ets \
                if sorted(a.annot["sample"])[0] == mutant ])

            if num == 6 and isabc(mutant): #two mutants - add replicate number
                mutant += "." + sorted(et.annot["replicate"])[0]            

            aw = mutants.get(mutant, [])
            aw.append(et)
            mutants[mutant] = aw

    return mutants

def createVectorGsea(mutantets, name):
    """
    Create series of example tales for vectorised GSEA.
    Each example table specifies one time point.
    """

    #sort by tech replicate - for order of examples
    techReplicate = lambda x: list(x.annot["techReplicate"])[0]
    sets = sorted(mutantets, key=techReplicate)

    #create example table for each column:
    domain = orange.Domain( \
        [ orange.FloatVariable(a) for a in attributeValues(sets[0], "DDB") ], \
        orange.EnumVariable("sample", values=[name]))

    outets = []

    for col in range(len(sets[0].domain.attributes)):
        lvals = []

        for et in sets:
            vals = attributeValues(et, col, lambda x: x.native())
            lvals.append(vals + [ name ])
        
        outet = orange.ExampleTable(domain, lvals)
        outets.append(outet)
    
    return outets

def concatExampleTables(joinl, domain=None):
    """
    Joins multiple one-mutant vector gsea example tables.
    """

    def anames(data):
        return [ a.name for a in data.domain.attributes ]

    if len(set(([ ",".join(anames(j)) for j in joinl ]))) != 1:
        crash("Vsi attributi marajo biti enakih imen")

    if not domain:
        classvals = sorted(set(m.flatten([ j.domain.classVar.values for j in joinl ])))
        domain = orange.Domain(\
            joinl[0].domain.attributes, \
            orange.EnumVariable("sample", values = classvals))

    values = []
    for j in joinl:
        for ex in j:
            ev = exampleValues(j, ex, lambda x: x.native())
            values.append(ev)

    return orange.ExampleTable(domain, values)

def joinMutantsVG(joinl):
    """
    Joins example tables of multiple mutants.
    """
    classvals = sorted(set(m.flatten([ j[0].domain.classVar.values for j in joinl ])))
    domain = orange.Domain(\
        joinl[0][0].domain.attributes, \
        orange.EnumVariable("sample", values = classvals))

    return [ concatExampleTables(list(a), domain=domain) for a in zip(*joinl) ]

from math import sqrt
import orngTest, orngStat

def knn(data):
    learner = orange.kNNLearner(k=len(data))
    res = orngTest.crossValidation([ learner ], data)
    return orngStat.AUC(res)[0]

def groupdistances(data):
    g1 = []
    g2 = []

    c1 = data.domain.classVar.values[0]
    c2 = data.domain.classVar.values[1]

    for i,ex in enumerate(data):
        if ex[-1].value == c1:
            g1.append(i)
        else:
            g2.append(i)

    distance = orange.ExamplesDistanceConstructor_Euclidean(data)
    distmatrix = orange.SymMatrix(len(data))
    for i in range(len(data)):
        for j in range(i+1):
            distmatrix[i, j] = distance(data[i], data[j])

    def avdist(elements):
        dists = []
        for i in range(len(elements)):
            for j in range(i+1):
                dists.append(distmatrix[elements[i], elements[j]])
        return float(sum(dists))/len(dists)

    print " "
    for ex in data:
        print ex
    
    ores = avdist(range(len(data)))/(avdist(g1)+avdist(g2))
    print ores
    return ores

def printGseaRes(res):
    for k,v in sorted(res.items(), key=lambda x: x[1][2]):
        print k, v[:4]

def dm3(a):
    return "%0.3f" % a
    
def knnvec1(samplesG1=["abcG15-"], samplesG2=["abcC3-", "abcC13-" ], ver=defver, evaluator=knn, impute=False, **kwargs):
    #print samplesG1
    #print samplesG2

    ets = getBasicDataPck()
    samplesD = samplesDict(ets)
    #print samplesD
    
    outets = [ createVectorGsea(samplesD[s1], s1) for s1 in samplesG1+samplesG2 ]
    #print outets

    gen1 = unpckGS(pathwaysFile)
    net = joinMutantsVG(outets)


    import knnWPearson

    if impute:
        print "IMPUTING"
        net = [ knnWPearson.impute(nn) for nn in net ]

    import obiGsea

    res2 = obiGsea.runGSEA(net, organism="ddi", geneSets=gen1, minPart=0.05, rankingf=obiGsea.evaluateEtWith(evaluator), classValues=[samplesG1, samplesG2], **kwargs) 

    return res2


def getGseaResults():
    """
    Gsea between each pair of mutants
    """

    ets = getBasicDataPck()
    samplesD = samplesDict(ets)
    jclasses = samplesD.keys()

    out = []
    for i in range(len(jclasses)-1):
        for j in range(i+1, len(jclasses)):
            print jclasses[i], jclasses[j]
            res2 = knnvec1(samplesG1=[jclasses[i]], samplesG2=[jclasses[j]], n=100, permutation="genes")
            out.append((jclasses[i], jclasses[j], res2))

    return out

import time
def getGseaResultsBetween(jclasses, legend=None, **kwargs):
    out = []

    if not legend:
        legend = []
        for l in jclasses:
            legend.append("+".join(l))

    needToDo = (len(jclasses)*(len(jclasses)-1))/2
    print "needToDo", needToDo
    status = 0

    for i in range(len(jclasses)-1):
        for j in range(i+1, len(jclasses)):
            print jclasses[i], jclasses[j], status, "/", needToDo
            times = time.time()
            res2 = knnvec1(samplesG1=jclasses[i], samplesG2=jclasses[j], permutation="genes", **kwargs)
            print time.time() - times
            out.append((legend[i], legend[j], res2))
            status += 1

    return out


def bufferedGseaResults():
    return m.autoPck("gsear_obiDic.v1", "3", getGseaResults)

def gseaSimilarity(gs):
    """
    Calculation of similarity by GSEA.
    """

    allMutants = sorted(set(m.nth(gs,0) + m.nth(gs,1)))
    print allMutants

    mid = dict([ (e,i) for i,e in enumerate(allMutants) ])

    mat = orange.SymMatrix(len(allMutants))

    def signifGenesets(m1,m2,res):
        res = sorted(res.items(), lambda x,y: cmp(x[1][2], y[1][2]))
        return len([ a for a,b in res if b[2] < 0.05 ])

    def newGenes(genes, genesets):
        for g in genesets:
            gen1 = set(g)
            gen2 = set(genes)
            dist = float(len(gen1 & gen2)) / len(gen1 | gen2)
            if dist > 0.8:
                return False
        return True


    def signifGenesets2(m1,m2,res):
        """
        Disregard similar similar genesets.
        """

        res = sorted(res.items(), lambda x,y: cmp(x[1][2], y[1][2]))
        genesets = []
        ql = []

        for a,b in res:
            if b[2] < 0.05:
                if newGenes(b[6], genesets):
                    ql.append(a)
                    genesets.append(b[6])
        return len(ql)

    for m1,m2,res in gs:
        mat[mid[m1],mid[m2]] = signifGenesets2(m1,m2,res)
 
    return mat, allMutants, mid

def MDSGseaSimilarity(mat, allMutants, mid, connect=False):

    mds = MDS(mat)

    print mat

    points = []
    import pylab
    for i,d in enumerate(allMutants):
        points.append((mds.points[i][0], mds.points[i][1], d))

    if connect:
        #connect near points
        for i in range(len(mid)-1):
            for j in range(i+1, len(mid)):
                if mat[i,j] <= 1:
                    pylab.plot([mds.points[i][0],mds.points[j][0]], \
                    [mds.points[i][1],mds.points[j][1]])

    replicatesGraph(points)
    clusterGseaSimilarity(mat, allMutants, mid)

def clusterGseaSimilarity(mat, allMutants, mid):
    hclust(mat, labels=allMutants, width=300)
    mat.objects = allMutants
    clus = orange.HierarchicalClustering(mat, linkage=orange.HierarchicalClustering.Average) 
    print printClustering(clus)

def extractMutant(label):
    """
    Extract mutant from label: transform to lowercase,
    remove everything following dot, remove abc, 
    remove all not alphanumeric characters.
    """
    l = label.lower()
    l = l.replace("abc", "")
    try:
        l = l[:l.index(".")]
    except ValueError:
        pass
    
    notanum = set([ c for c in l if not c.isalnum() ])
    for c in notanum:
        l = l.replace(c, "")

    return l

def phenotypeAssign(mut, selphen):
    """
    Returns mutant phenotype.
    """

    etsel = selphen
    notanum = set([ c for c in etsel if not c.isdigit() ])
    for c in notanum:
        etsel = etsel.replace(c, "")

    ctime = int(etsel)

    join = False
    if selphen[-1] == "G":
        join = True
        selphen = selphen[:-1]

    sys.path.append("./phenotypes/")
    import addPhenotypes as phent
    mutants,ptypes,dic = phent.loadPhenotypes("phenotypes/phenotypes.txt")

    mut = extractMutant(mut)

    if mut not in mutants:
        raise Exception("Possible mutants: " + str(mutants))
    if selphen not in ptypes:
        raise Exception("Possible selphen: " + str(ptypes))

    value = dic[(mut,selphen)]
    if not join:
        values = phent.getGroup2(value.split(','), ctime)
    else:
        values = phent.getGroup2J(value.split(','), ctime)

    return values

def gseaBetweenPhenotypes(selphen=None, onlyabc=True, **kwargs):

    ets = getBasicDataPck()
    samplesD = samplesDict(ets, onlyabc)
    jclasses = samplesD.keys()

    if selphen != "None":
        newPhens = dict([(c,phenotypeAssign(c, selphen)) for c in jclasses])
    else:
        newPhens = dict([(c,extractMutant(c)) for c in jclasses ])

    phevs = sorted(set(newPhens.values()))
    counts = [ (a, newPhens.values().count(a)) for a in phevs ]

    groups = [ [ mut for mut,phen in newPhens.items() if phen == phev ] for phev in phevs ]

    nameGroup = lambda g: "+".join(sorted([extractMutant(a) for a in g]))
    groupsmut = [ nameGroup(group) for group in groups ]

    combres = getGseaResultsBetween(groups, [ a + " " + b for a,b in zip(groupsmut,phevs)], **kwargs)
    return combres, counts

def bufferedGseaBetweenPhenotypes(selphen=None, **kwargs):
    ver = "1"
    print selphen
    return m.autoPck("gsear_obiDic_bp_s%s.v%s" % (selphen, ver), "3", gseaBetweenPhenotypes, selphen, **kwargs)

def applyToDm(dm, fn):
    dmout = orange.SymMatrix(len(dm[0]))
    for i in range(0, len(dm[0])):
        for j in range(0, i+1):
            dmout[i,j] = fn(dm[i,j])
    return dmout


from OWWidget import *
import orngTree, OWGUI
import orngEnsemble
from exceptions import Exception

class OWChooser(OWWidget):
    
    def __init__(self, parent=None, signalManager = None, name='Choose phenotypes', phenotypes=[], brg=None, ow=None, allres=None,  *args, **kwargs):
        OWWidget.__init__(self, parent, signalManager, name)

        self.args = args
        self.kwargs = kwargs
        self.brg = brg
        self.ow = ow
        self.firstRun = True

        self.pt1 = 0
        self.pt2 = 1
        self.ot = 0
        self.pvall = 0.05

        self.ptypes = []

        self.allres = allres

        self.types = OWGUI.comboBoxWithCaption(self.controlArea, self, "ot", items=sorted(allres.keys()), label="Observed phenotypes", callback=self.oc)
        self.ph1 = OWGUI.comboBoxWithCaption(self.controlArea, self, "pt1", items=self.ptypes, label="First phenotype", callback=self.sc)
        self.ph2 = OWGUI.comboBoxWithCaption(self.controlArea, self, "pt2", items=self.ptypes, label="Second phenotype", callback=self.sc) 

        kernelSizeValid = QDoubleValidator(self.controlArea)
        kernelSizeValid.setRange(0,1,3)

        self.pvalue = OWGUI.lineEdit(self.controlArea, self, "pvall", label="P value", valueType=float, validator=kernelSizeValid, callback=self.sc)
 
        self.oc()
        self.sc()

    def oc(self):
        self.optype = sorted(self.allres.keys())[self.ot]
        print "observer phenotypes", self.optype
        self.ptypes = [ str(a) for a in self.allres[self.optype][1] ]
        print self.ptypes
        phenotypes = self.ptypes
        self.ph1.clear()
        self.ph2.clear()
        self.ph1.addItems(self.ptypes)
        self.ph2.addItems(self.ptypes)
        self.pt1 = 0
        self.pt2 = 1
        self.sc()
    
    def sc(self):
        if self.pt1 == self.pt2:
            self.pt2 = (self.pt2 + 1) % len(self.ptypes)

        print self.optype

        ind = linearizedPosition(self.pt1, self.pt2, len(self.ptypes))
        print self.allres[self.optype][0][ind][:2]

        allres2 = limitPValue(self.allres, self.pvall)

        showGseaNetwork(allres2[self.optype][0][ind][2], self.ow, self.firstRun, *self.args, **self.kwargs)
        self.firstRun = False

def linearizedPosition(i, j, size):
    if j < i:
        t = i
        i = j
        j = t
    #withFullLines = i*(2*(size-1)+(i-1)*(-1))/2 #arithmetic sum
    #inThisLine = j - (i+1)
    #return withFullLines + inThisLine
    #return i*(2*size-1-i)/2 + j - i - 1
    return i*size - i*(1+i)/2 + j - i - 1


def initWidgets(brg, count, allres, *args, **kwargs):
    sys.path.append("/usr/lib/python2.5/site-packages/orange/OrangeWidgets/Unsupervised")
    sys.path.append("c:/Python25/lib/site-packages/orange/OrangeWidgets/Unsupervised")

    import OWNetwork as own
    appl = own.QApplication(sys.argv)
    ow = own.OWNetwork()
    ow.optMethod = 2

    ow.invertSize = 1
    ow.maxVertexSize = 15

    ow.show()

    owc = OWChooser(phenotypes=count, brg=brg, ow=ow, allres=allres, *args, **kwargs)
    owc.show()

    appl.exec_()


def exportET(resl):
    #do not sort them inside
    
    if len(resl) <= 0:
        return None

    def splitn(name):
        splitndx = name.find("]")
        collection = name[1:splitndx]
        name = name[splitndx + 1:]
        return collection,name

    allCollections = sorted(set([splitn(name)[0] for name,_ in resl]))

    vars = []
    vars.append(orange.StringVariable("sname"))
    vars.append(orange.StringVariable("Name"))
    vars.append(orange.EnumVariable("Collection", values=allCollections ))
    vars.append(orange.FloatVariable("NES"))
    vars.append(orange.FloatVariable("ES"))
    vars.append(orange.FloatVariable("P-value"))
    vars.append(orange.FloatVariable("FDR"))
    vars.append(orange.StringVariable("Geneset size"))
    vars.append(orange.StringVariable("Matched size"))
    vars.append(orange.StringVariable("Genes"))

    domain = orange.Domain(vars, False)

    examples = []
    for name, (es, nes, pval, fdr, os, ts, genes) in resl:
        collection, name = splitn(name)
        sname = collection + name
        if collection[:2] == "GO":
            sname = collection + " " + " ".join(sname.split(" ")[3:])
            print sname
        examples.append([sname, name, collection, nes, es, pval, min(fdr,1.0), str(os), str(ts),  ", ".join(genes)])

    return orange.ExampleTable(domain, examples)


def exportDistanceMatrix(resl):
    """
    Input: results as a list of tuples
    """

    dm = orange.SymMatrix(len(resl))

    for i in range(len(resl)-1):
        for j in range(i+1, len(resl)):
            gen1 = set(resl[i][1][6])
            gen2 = set(resl[j][1][6])
            dm[i,j] = float(len(gen1 & gen2)) / len(gen1 | gen2)

    return dm


def showGseaNetwork(res, ow, firstRun, limit=0.3, limitpvalue=0.1, fn=lambda x: x, phen=[], networkStays=True):
    """
    Show GSEA results on graph
    """
    sys.path.append("/usr/lib/python2.5/site-packages/Genomics/widgets/")
    et = exportET(res.items())
    
    #print et[0]
    #print et[1]

    print limitpvalue

    def changeEverything():
            print "changing everything"
            from orangeom import Network
            dm = exportDistanceMatrix(res.items())
            dm = applyToDm(dm, fn)
            graph = Network(len(et), False)
            nedges = graph.fromSymMatrix(dm, fn(limit), 1.0)
            graph.items = et
            ow.setGraph(graph)

            ow.vertexSize = 3
            ow.setVertexSize()

    if networkStays:
        if firstRun:
            changeEverything()
        else:
            ow.setItems(et)
    else:
        if 1:
            changeEverything()




def geneToTerms():

    import go
    go.loadGO() 
    go.loadAnnotation('dictyBase')
    print "Annot loaded"
    terms = [t.id for t in go.loadedGO.terms ]
    

    count = 0
    def cb():
        print count
        count += 1

    print "Find genes"

    #gene -> term
    gt = go.findGenes(GOTerms=terms, reportEvidence=False, directAnnotationOnly=False, progressCallback=cb)
    print gt.items()[0]
    return gt

def searchKEGG(genenames):
    """
    Analyses how are genes represented in kegg database.
    """
    import obiKEGG
    kegg = obiKEGG.KEGGOrganism("ddi")

    listunk = []
    for gene in genenames:
        if isinstance(gene, basestring):
            a,b,c = kegg.get_unique_gene_ids([gene], caseSensitive=False)
        else:
            for g in gene:
                a,b,c = kegg.get_unique_gene_ids([g], caseSensitive=False)
                if len(a) > 0:
                    break

        if len(a) == 0:
            listunk.append(gene)

    print len(listunk), listunk 

def ddiPathways():

    gen1 = {}

    def addSource(dic, addition):
        return dict( \
            [ (addition + name,genes) for name,genes in dic.items() ]\
            )

    # KEGG PATHWAYS

    import obiKEGG
    kegg = obiKEGG.KEGGOrganism("ddi")

    pways = kegg.list_pathways()

    dicp = {}
    for id,desc in pways.items():
        dicp[desc] = kegg.get_genes_by_pathway(id)

    gen1.update(addSource(dicp, "[KEGG] "))

    # GO pathways

    import go
    go.loadGO()
    go.loadAnnotation('dictyBase')
 
    #gt = geneToTerms()
    gt = m.autoPck("geneToTermsT.pck", "4", geneToTerms)
    
    print "gene to terms", len(gt)

    td = go.loadedGO.termDict

    map = {} #map from set id to to it's genes

    print "POS KEYS", len(gt.keys())

    for gene,sets in gt.items():
        for s in sets:
            cs = map.get(s, set())
            cs.add(gene)
            map[s] = cs

    print len(map)

    nmap = {}

    def getDDB(l):
        """ spremeni geneDDB v DDB" """
        for a in l:
            try:
                return a[a.index("DDB"):]
            except:
                pass
        return l[0]

    aspectmap = { "biological_process":"P", "molecular_function":"F", "cellular_component":"C" }

    for a,b in map.items():
        #print a, td[a].name, len(b)

        lg = [ getDDB([g] + go.loadedAnnotation.geneNamesDict[g]) \
            for g in list(b) ]

        try:
            aspect = aspectmap[go.loadedGO.termDict[a].aspect]
            nmap[a + " " + td[a].name ] = lg, aspect
        except: 
            print "Unsucessful", a

    print len(nmap)

    #test for gene names

    genenames = list(set(reduce(lambda x,y: x+y, [list(a) for a in map.values()])))

    allaliases = [ go.loadedAnnotation.geneNamesDict[name] for name in genenames ]

    genenameswddb = [ getDDB(als) for als in allaliases if getDDB(als) != False ]
    print len(genenameswddb)
    genenames = genenameswddb

    #print genenames[:100]

    print "SEARCH KEGG", "all", len(genenames) ,"notin", searchKEGG(genenames)
    
    for a,(b,asp) in nmap.items():
        gen1[("[GO %s]" % asp) + " " +  a] = b

    return gen1

def pathwayAnalysis(outfile="precompPathways2.pck"):

    ets = getBasicDataPck()

    def names():
        return [ ex["DDB"].native() for ex in ets[0] ]

    genenames = names()
    pways = ddiPathways()

    import obiGeneMatch
    gm = obiGeneMatch.GeneMatch(genenames, caseSensitive=False, organism="ddi")

    print "Gene values", len(set(m.flatten(pways.values())))

    gen1 = {}

    for pathw in pways.items():
        matched = gm.match(pathw[1])
        if len(matched) > 3:
            print pathw[0], matched, len(matched), len(pathw[1])
            gen1[pathw[0]] = pathw[1]

    print "NUM", len(gen1)

    m.pck(outfile, gen1)
    return gen1


def demonstration1():
    """
    Demonstration as shown to Gady on 22.7.2008 
    """
    relphen = [ "12Hrs", "18Hrs", "24Hrs", "12HrsG", "18HrsG" ]

    allres = dict([ (a,bufferedGseaBetweenPhenotypes(a, n=100)) for a in relphen ])
    bgr, counts = allres["12Hrs"]
    print counts
    sel = 1
    n1, n2, res = bgr[sel]
    print n1, n2

    initWidgets(bgr, counts, allres, limit=0.3, fn=lambda x: x, phen=counts)

def limitPValue(allres, pvallimit):
    allres2 = {}
    for name,resl in allres.items():
        bgr, counts = resl
        nbgr = []
        for n1,n2,res in bgr:
            print "here", n1," , ", n2
            res2 = dict([ (gsname,r) for gsname,r in res.items() if r[2] < pvallimit ])
            #print dict([ (gsname,r[:3]) for gsname,r in res.items() ])
            #pvaldists.append([ r[2] for gsname,r in res.items() ])
            nbgr.append((n1,n2,res2))
        allres2[name] = (nbgr, counts)
    return allres2

def demonstration2():
    """
    Demonstration as shown to Gady on 22.7.2008 
    """
    relphen = [ "None" ]
    print relphen
    
    import knnWPearson

    def knnp(data):
        learner = orange.kNNLearner(k=len(data), distanceConstructor=knnWPearson.PearsonDistanceConstructor)
        res = orngTest.crossValidation([ learner ], data)
        return orngStat.AUC(res)[0]

    def ne(data):
        """
        squared knn score
        """
        return knn(data)**2

    evaluator = None
    #evaluator = knnp

    if defver[:4] == "pka2":
        print "changing evaluator"
        evaluator = ne

    impute=False

    def gr():
        return dict([ (a,gseaBetweenPhenotypes(a, onlyabc=False, n=100, atLeast=0, evaluator=evaluator, impute=impute)) for a in relphen ])

    allres = m.autoPck("dem2.%s.pck" % hash(defver), 2, gr)
    #allres = gr()
    
    """
    import pylab,math
    pylab.hist([-math.log(max(p,0.0005),10) for p in  pvaldists[0] ], bins=20)
    pylab.show()
    """
    #allres = limitPValue(allres, 0.1)

    bgr, counts = allres[relphen[0]]
    print counts
    sel = 0
    n1, n2, res = bgr[sel]
    print n1, n2

    initWidgets(bgr, counts, allres, limit=0.3, fn=lambda x: x, phen=counts,  networkStays=False)

pathwaysFile = "precompPathways2.pck" # abc pathways

#defver = "pka2 [ 'pkaC-', 'regA-' ]"
#defver = "pkar [ 'pkaC-', 'regA-' ]"
#defver = "pkar [ 'pkaC-', 'yakA-', 'acaA-' ]  "
defver = "pkar [ 'RegA-pspsA', 'acaA-', 'acaA- pkaCO/E', 'pkaC-', 'pkaR-', 'pkaR-regA-', 'pufA-', 'pufA-pkaC-', 'pufA-pkaR-', 'regA-', 'yakA-', 'yakA-pufA-', 'ax4'] "
#defver = "pkar [ 'acaA-', 'pkaC-', 'pkaR-', 'pufA-', 'regA-', 'yakA-', 'ax4']"


pathwaysFile = "precompPathways5.pck" # more pathways

#ets = getBasicDataPck()
#print ets[0][0]

gen1 = unpckGS(pathwaysFile)

"""
samplesD = samplesDict(ets)
outets = [ createVectorGsea(samplesD[s1], s1) for s1 in samplesD.keys() ]
normalgsea = concatExampleTables(outets[0] + outets[1])
print "NORMAL"
res3 = obiGsea.runGSEA(normalgsea, n=100, organism="ddi", geneSets=gen1, atLeast=3, minPart=0.05, permutation="genes") 
dsafsda()
"""
demonstration2()

if __name__=="__main__":

    import optparse
    p = optparse.OptionParser()
    p.add_option("--gseasim", action="store_true", help="Gsea as time vector similirity analysis.")
    p.add_option("--mdsgseasim", action="store_true", help="Gsea as time vector similirity analysis: MDS.")
    p.add_option("--v1", action="store_true",
        help="Bigger table")

    (opts, args) = p.parse_args()

    ver = 2
    if opts.v1:
        ver = 1

    fn = None
    if len(args) >= 1:
        fn = args[0]

    if opts.gseasim: print gseaSimilarity(bufferedGseaResults())
    if opts.mdsgseasim: print MDSGseaSimilarity(gseaSimilarity(bufferedGseaResults()))


