import sys, pprint, time, re
import mMisc as m
from itertools import *
import urllib
import orange

import socket

socket.setdefaulttimeout(10)

class HttpGetException(Exception): pass

def httpGet(address, *args, **kwargs):
    verbose = 1
    if verbose: 
        print address
    t1 = time.time()
    f = urllib.urlopen(address, *args, **kwargs)
    read = f.read()
    if verbose:
        print time.time() - t1
    return read

def txt2ll(s, separ=' ', lineSepar='\n'):
    return [ a.split(separ) for a in s.split(lineSepar) ]

class DBInterface(object):
 
    def __init__(self, address):
        self.address = address

    def raw(self, request, tryN=3):
        print "tryN", tryN

        if tryN == 0:
            return None

        try:
            return httpGet(self.address + request)
        except:
            return self.raw(request, tryN=tryN-1)

    def get(self, request):
        rawf = self.raw(request)
        a = txt2ll(rawf, separ='\t')
        if a[-1][0] == "":
            a = a[:-1]
        return a

def _test():
    import doctest
    doctest.testmod()

def splitTableOnColumn(ll,n):
    omap = {}
    n = m.lloc(ll,n)
    for l in ll:
        cell = omap.get(l[n], [])
        cell.append(l)
        omap[l[n]] = cell
    return omap

def neededColumns(legend, want):
    return [ legend.index(a) for a in want ]

def onlyColumns(ll, legend, want):
    return list(m.imnth(ll, neededColumns(legend, want)))

def ll2dic(ll, key, value):
    """
    Converts LL to map. Key is key position, value value position
    """
    names = m.nth(ll, m.lloc(ll, key))
    if len(names) == len(set(names)):
        return dict(list(m.imnth(ll, [ m.lloc(ll, key),  m.lloc(ll, value) ] )))
    else:
        raise Exception("all keys are not unique")

def dic2ll(dic):

    columns = sorted(dic.values()[0].keys())
    ll = []
    for key, d in dic.items():
        ll.append([ key ] + [ d[a] for a in columns ])
    return columns,ll

def unique(els):
    return len(els) == len(set(els))

def reverseDic(d):
    """
    Create a reverse dictionary if a unique reverse is possible.
    """
    if unique(d.values()):
        return dict([ (b,a) for a,b in d.items() ])

def chainLookup(a, dics, force=[]):
    """
    Goes through a list of dictionaries. On each step try to
    transform current key to value. If unsuccessfull, query the next
    dictionary with current key. Force last translation.
    """
    force = [ m.loc(dics, i) for i in force ]
    for i,dic in enumerate(dics):
        if i in force: #force specified translations
            a = dic[a]
        else:
            if a in dic:
                a = dic[a]
    return a

class DatabaseConnection(object):

    """
    Type is object id
    """
    
    aoidPairs = txt2ll("""timePoint extractions.developmental_time_point
sample biological_samples.sample
growthCond biological_samples.growth_condition
treatment biological_samples.treatment
replicate biological_sample_replicates.replicate
techReplicate chips.replicate
platform chips.chip_platform
isTimeSeries biological_sample_replicates.is_time_series""")

    obidPairs = txt2ll("""norms normalizations
samples biological_samples
replicates biological_sample_replicates
analysis analysis
experiments experiments
extractions extractions
chips chips""")

    def __init__(self, address):
        self.db = DBInterface(address)
        self.preload()

    def preload(self):

        # aoids are mappings from annotation name to annotation id
        self.aoids = ll2dic(self.__annotationTypes(), 1, 0)
        self.saoids = ll2dic(self.aoidPairs, 0, 1)
        self.aoidsr = reverseDic(self.aoids)
        self.saoidsr = reverseDic(self.saoids)

        # obids are mappings from object id to annotation id
        self.obids = ll2dic(self.__objects(), 1, 0)
        self.sobids = ll2dic(self.obidPairs, 0, 1)
        self.obidsr = reverseDic(self.obids)
        self.sobidsr = reverseDic(self.sobids)

    def aoidt(self, s):
        return chainLookup(s, [self.saoids, self.aoids], force=[-1])

    def obidt(self, s):
        return chainLookup(s, [self.sobids, self.obids], force=[-1])
 
    def aoidtr(self, s, **kwargs):
        return chainLookup(s, [self.aoidsr, self.saoidsr], **kwargs)

    def obidtr(self, s):
        return chainLookup(s, [self.obidsr, self.sobidsr])

    def pq(self, q):
        """
        Prepare query
        """

        def createQuery(l):
            return "&".join([ self.aoidt(a) + "=" + b for a,b in l.items()])

        q = createQuery(q)
        q = q.replace("=","***")
        q = q.replace("&","|||")
        return q

    def downloadMulti(self, command, ids):
        """
        Results in the same order as inputed!
        """
        sids = m.split(ids,100)

        for i,sidp in enumerate(sids):
            res = self.db.get(command.replace("$MULTI$", ",".join(sidp)))

            legend = res[0]
            res = res[1:]

            #split on different values of the first column - first attribute
            antss = splitTableOnColumn(res, 0)
    
            #put results in order
            tl = []
            for ci in sidp:
                yield antss[ci], legend

    def geneInfo(self):
        res,legend = self.sq("action=gene_info")
        return res, legend

    def annotationOptions(self, aoid=None, onlyDiff=False, **kwargs):
        params = ""
        if len(kwargs) > 0: params += "&query=" + self.pq(kwargs)
        if aoid: params += "&annotation_object_id=" + str(aoid)
        res,legend = self.sq("action=get_annotation_options%s" % (params))
        res = onlyColumns(res, legend, ['annotation_object_id', 'value' ])
        #join columns with the annotation object id
        joined = {}
        for key,v in res:
            key = self.aoidtr(key)
            cur = joined.get(key, [])
            cur.append(v)
            joined[key] = cur

        if onlyDiff:
            joined = dict([ (a,b) for a,b in joined.items() if len(b)>1 ])

        return joined

    def sq(self, s1):
        res = self.db.get(s1)
        return res[1:],res[0]

    def annotation(self, type, id):
        return list(self.annotations(type, [ id ]))[0]

    def annotations(self, type, ids):
        antss = self.downloadMulti( \
            "action=get_annotations&ids=$MULTI$&object_id=%s" \
            % (self.obidt(type)), ids)

        for ants in antss:
            res, legend = ants
            res2 = onlyColumns(res, legend, ['name', 'value'])
            res2 = [ [ self.aoidtr(a),b ] for a,b in res2 ]
            yield res2

    def search(self, type, **kwargs):
        res,_ = self.sq("action=search&object_id=%s&query=%s" \
            % (self.obidt(type), self.pq(kwargs)))
        return m.nth(res, 0)

    def chipN(self, id):
        res,legend = self.sq("action=get_normalized_data&ids=%s" % (id))
        res2 = onlyColumns(res, legend, ["spot_id", 'M', 'A'])
        return res2

    def chipR(self, id):
        res,legend = self.sq("action=get_raw_data&ids=%s" % (id))
        #TODO remove unneeded columns
        res2 = res
        return res2
  
    def chipNs(self, ids):
        for id in ids:
            yield self.chipN(id)

    def chipRs(self, id):
        for id in ids:
            yield self.chipR(id)
   
    def spotId(self):
        res,legend = self.sq("action=spot_id_mapping")
        res2 = onlyColumns(res, legend, ["spot_id", 'ddb', 'genename'])
        return res2

    def annotationTypes(self):
        return self.aoids.keys()

    def objects(self):
        return self.obids.keys()

    def __annotationTypes(self):
        """
        Returns list of [ annotation_object_id, name ]
        """
        res, legend = self.sq("action=get_annotation_types")
        res2 = onlyColumns(res, legend, ["annotation_object_id", 'name'])
        return res2

    def __objects(self):
        res, legend = self.sq("action=get_objects")
        res2 = onlyColumns(res, legend, ["object_id", 'object_name'])
        return res2

    def spotMap(self):
        spotids = self.spotId()
        spotmap = dict([ (a[0],a[1]) for a in spotids])
        return spotmap

    def allAnnotationVals(self, annots):
        """
        All annotation valuess for given annotations
        in a dict of { name: set of possible values } pairs.
        """
        av = {}
        for a in annots:
            for name,val in a:
                cvals = av.get(name, set([]))
                cvals.add(val)
                av[name] = cvals
        return av

    def selectN(self, **kwargs):
        """
        Return M measurements of the normalized data. 

        Returns an example table.
        """
        type = 'norms'

        ids = self.search(type, **kwargs)
        spotmap = self.spotMap()

        read = []
        for annot,chip in izip(self.annotations(type, ids), self.chipNs(ids)):
            mapped = [ [ spotmap.get(a, "#"+a) ] for a in m.nth(chip, 0) ]
            chip = list(m.ihj(m.imnth(chip, [0,1]), mapped))
            # for each chip I how have: spotid, M, ddb
            read.append((annot, chip))

        return read

    def createExampleTables(self, read, prefix="", allowDiff=[], meaningless=[ ".*filename.*", "chips.name" ], ignore = [], ignoreNot = []):
        """
        From annotation, chip reading pair (spotid, val, ddb)
        create a series of example tables.
        """

        if len(read) == 0:
            return None

        #allowDiff should be top most specification
        topMost = [ self.aoidtr(a) for a in self.aoids.keys() ]
        
        def forceTopMost(l):
            for a in l:
                if a not in topMost:
                    raise Exception(a + " not top most annotation type." )
 
        forceTopMost(allowDiff)

        #all possible annotations
        annots = self.allAnnotationVals(m.nth(read,0))

        ignore = meaningless + ignore
        ignoreNot = ignoreNot + allowDiff

        def isIgnored(a):
            for exp in ignoreNot:
                if re.match(exp, a):
                    return False
            for exp in ignore:
                if re.match(exp, a):
                    return True
            return False
    
        def isMeaningless(a):
            for exp in ignoreNot:
                if re.match(exp, a):
                    return False
            for exp in meaningless:
                if re.match(exp, a):
                    return True
            return False

        def ok(x):
            a,b = x
            if len(b) <= 1:
                return False
            if isIgnored(a):
                return False
            return True

        diffannots = dict(filter(ok, annots.items()))
        #print "DIFF ANNOTS", diffannots

        """
        Need to group
        those which have everything the same except of allowDiff while
        ignoring annotation which match meaningless + ignore.
        """
    
        def validAnnot(x):
            try: 
                print self.aoidt(x)
                return True
            except:
                return False

        #allowDiff = [ "timePoint" ]
        allowDiffSet = set(filter(validAnnot, allowDiff))

        danames = set(diffannots.keys())
        reallyDiff = danames & allowDiffSet

        #print "ALLOWDIFF", allowDiff

        leaveKeys = danames - reallyDiff
        #print "LEAVEKEYS", leaveKeys

        #remove all non-neccessary keys form the annotation dictionary
        #and form tem in groups afterwards

        def dictWithOnly(dic, keys):
            """
            Returns a shallow copy of the dictionary with
            key not in keys removed.
            """
            odic = {}
            for a,b in dic.items():
                if a in keys:
                     odic[a] = b
            return odic

        candidates = [ dictWithOnly(dict(d),leaveKeys) \
            for d in m.nth(read, 0) ]

        def groupDicts(ldics):
            """
            Group same dictionaries together. Returns idices
            of original dictionaries
            """
            ldics = [ frozenset(d.items()) for d in ldics ]
            dd = {}
            for i,d in enumerate(ldics):
                group = dd.get(d, [])
                group.append(i)
                dd[d] = group
            return dd.values()

        groups = groupDicts(candidates)
        #FIXME - order those groups
        
        """
        At this point the groups going to the same example table
        are already made. Now we need to control value order, example
        order and table order for those groups.

        Important decisions:
        - example order: by spot id
        - attribute order: ([1,2,3],[1,2]) -> [1,1],[1,2],[2,1],...
        - table order - NOW RANDOM - LEAVE IT SO?
        """

        def csorted(l):
            """
            Return sorted list of string , but choose sorting type first:
            if all are integer, sort in integers and then return string
            """
            try:
                l = [ int(a) for a in l ]
            except:
                pass
            return [ str(a) for a in sorted(l) ]

        #respect order from allowDiff
        differentialNames = [ a for a in allowDiff if a in reallyDiff ]
        differentialVals = [ csorted(diffannots[a]) for a in differentialNames ]

        def difV(li):
            """
            Returns value for tuple of indices
            """
            return [ differentialVals[i][v] for i,v in enumerate(li) ]

        def nameInds(values, inds):
            """
            Create attribute name for given values.
            """
            def aux(i,a):
                name = '_'.join([ n + '=' + v for n,v in values ])
                if len(inds) > 1:
                    name += "_" +  str(i)
                return name

            return [ aux(i,a) for i,a in enumerate(inds) ]
               
        exampleTables = []

        chips = m.nth(read,1)

        #where to put n-th occurance of some spot id
        def outPositionMap(ids):
            """
            Where to put n-th occurance of some spot id.
            Input: list of lists of ids.
            Output: map of id,repetition -> position
            """
            
            amap = {}

            for lids in ids:
                print len(lids)
                repeats = {}
                for id in lids:
                    rep = repeats.get(id, 0)
                    repeats[id] = rep+1
                    key = (id, rep)
                    amap[key] = amap.get(key, 0) + 1

            return dict( [ (k,i) for i,k in enumerate(sorted(amap.keys())) ] )

        posMap = outPositionMap([ m.nth(c,0) for c in chips ])
        npos = len(posMap.values())

        ddb = [ "" for a in range(npos) ] 
        spotmap = self.spotMap()
        for k,ind in posMap.items():
            ddb[ind] = spotmap.get(k[0], "#"+k[0]) 

        #print "GROUPS", groups

        for group in groups: #FIXME debug

            lens = [ len(a) for a in differentialVals ]
            selected = []

            gannots = [ ll2dic(m.nth(read, 0)[a], 0, 1) for a in group ]

            validgannots = [ dict(filter(lambda a: not isMeaningless(a[0]), \
                gan.items())) for gan in gannots ]
            vanv = self.allAnnotationVals([ v.items() for v in validgannots ])

            #output a set off possible values
            commonAnnots = vanv

            groupnames = []
            groupvals = []

            #generate column values and names for the whole example table

            for valsi in m.mxrange([ b for b in lens ]):

                needValues = zip(differentialNames, difV(valsi))
                inds = extractNeeded(gannots, needValues)

                names = nameInds(needValues, inds)
                valuel = []

                cinds = [ (ind,group[ind]) for ind in inds ]

                #set values for that column in example table

                for ind,cind in cinds:

                    vals = [ None ] * npos

                    repeats = {}
                    putinds = []
                    for id,v,gene in chips[cind]:
                        rep = repeats.get(id, 0)
                        repeats[id] = rep+1
                        key = (id, rep)
                        putind = posMap[key]
                        putinds.append(putind)
                        vals[putind] = v

                    valuel.append(vals)

                groupnames = groupnames + names
                groupvals = groupvals + valuel

            """
            Create example table and add it's annotation.
            Which annotation to add? 
            1. Those that are the same for their group. NOT NOW
            2. Every one.
            3. Those ignored?

            For now add every annotation.
            """
            et = createExampleTable(groupnames, groupvals, ddb)
            et.annot = dict([ (a,list(b)) for a,b in commonAnnots.items() ])
            exampleTables.append(et)
    
        return exampleTables

def createExampleTable(names, vals, ddb):
    """
    Create an ExampleTable for this group. Attributes are those in
    names
    """
    attributes = [ orange.FloatVariable(n, numberOfDecimals=3) \
        for n in names ]
    domain = orange.Domain(attributes, False)
    ddbv = orange.StringVariable("DDB")
    id = orange.newmetaid()
    domain.addmeta(id, ddbv)

    examples = []
    for v,d in izip(izip(*vals), ddb):
        ex = orange.Example(domain, [ floatOrUnknown(a) for a in v ])
        #ex = orange.Example(domain, list(v))
        ex["DDB"] = d
        examples.append(ex)

    return orange.ExampleTable(domain,examples)

   
def extractNeeded(annotations, needValues):
    """
    Returns indices of annotation with all values as specified in
    needValues.
    """
    ok = []
    for i,an in enumerate(annotations):
        sl = sum([ an[nam] != val for nam,val in needValues ])
        if sl == 0: 
            ok.append(i)
    return ok


def median(l):
    if len(l) == 0:
        return None
    l = sorted(l)
    if len(l) % 2 == 1: #odd
        return l[len(l)/2]
    else: #even
        return (l[len(l)/2-1] + l[len(l)/2])/2.0


def averageAttributes(data, join="DDB", fn=median):
    """
    Averages attributes with the same "join" parameter using
    specified function. Builds a now dataset. Order of
    "join" parameter stays the same with regard to first
    appearance.
    """

    valueso = []
    valuess = set(valueso)

    attributes = [ a for a in data.domain.attributes ]
    domain = data.domain

    #accumulate values
    valuesddb = dict( [ (at,{}) for at in attributes ])

    for ex in data:
        #join attribute - ddb
        j = str(ex[join])

        if j not in valuess:
            valueso.append(j)
            valuess.add(j)

        for a in attributes:
            val = ex[a]
            l = valuesddb[a].get(j, [])
            if not val.isSpecial():
                l.append(val.native())
            valuesddb[a][j] = l

    #print "len valueso", len(valueso)
    #print sorted(set([ str(ex[join]) for ex in data ]))

    #apply function fn to each attribute

    #print valuesddb[valuesddb.keys()[0]]

    for a in attributes:
        for n,v in valuesddb[a].items():
            valuesddb[a][n] = floatOrUnknown(fn(v))

    #create a new example table reusing the domain
    examples = []
    for v in valueso:
        example = orange.Example(domain, \
            [ valuesddb[a][v] for a in attributes ] )
        example[join] = v
        examples.append(example)

    return orange.ExampleTable(domain, examples)

def floatOrUnknown(a):
    """
    Converts an element to float if possible.
    If now, output "?".
    """
    try:
        return float(a)
    except:
        return "?"

def run2():

    #_,dbc = m.autoPickler("tmpdbc.pck", "1", DatabaseConnection, "purple.bioch.bcm.tmc.edu/~anup/index.php?")
    dbc = DatabaseConnection("http://purple.bioch.bcm.tmc.edu/~anup/index.php?token=gFl0g7y5lefcnY6mqAdLd&")
    read = dbc.selectN(platform='minichip')
    et = dbc.createExampleTables(read, allowDiff=["timePoint"], ignore=[".*"], ignoreNot=["techReplicate", "sample"] )


if __name__=="__main__":

    run2()





