import orange

def nth(l, n):
    #only n-th element in list of lists structure
    return [ a[n] for a in l ]

def loadPhenotypes(fn):
    """
    returns all mutants, all phenotpyes types, and 
    dictionary from (mutant, phenotype type) -> type
    """
    s1 = open(fn, "rt").read()
    lines = s1.split("\n")

    #convert file to list of lists form
    ll = []

    for i,l in enumerate(lines):
        if len(l) > 0:
            tabs = l.split('\t')
            ll.append(tabs)

    phenNames = ll[0][1:]
    ll = ll[1:]

    mutants = [ a.lower() for a in nth(ll,0) ] #lower case for easier matching
    cols = []

    for coli in range(1, len(ll[0])):
        cols.append([ a.replace(" ", "") for a in nth(ll, coli) ])

    dic = {}

    for phen,types in zip(phenNames, cols):
        for mut,type in zip(mutants, types):
            #print phen, mut, type
            dic[(mut,phen)] = type

    return mutants, phenNames, dic

#first in each sublist is WT morphology, others are alternative
phentime = [ [ "STR" ], [ "LAG" ], ["MND", "YMND", "MTAG" ], 
                [ "TAG" ], [ "FNG", "MFNG", "YFNG" ], [ "SLG" ], 
                [ "MXH", "CUD" ], [ "EFB" ], [ "FBS", "GNR", "SFB", "MFBS", "LFBS" ], 
                [ "LFB", "YFB", "ASN" ] ]
#MODIFIED by Marko - put ECUL with EFB
phentime = [ [ "STR" ], [ "LAG" ], ["MND", "YMND", "MTAG" ], 
                [ "TAG" ], [ "FNG", "MFNG", "YFNG" ], [ "SLG" ], 
                [ "MXH", "CUD" ], [ "EFB","ECUL" ], [ "FBS", "GNR", "SFB", "MFBS", "LFBS" ], 
                [ "LFB", "YFB", "ASN" ] ]


times = range(8, 28, 2)
phenotypesAt = dict(zip(times, phentime))

def getGroup1(l, time):
    """
    Return phenotype from list of phenotypes. Check which of the phenotypes for 
    this mutant at that time is the closest to WT or alternative at that time.
    First look for more advances phenotypes (for example, +2h has priority over -2h).
    If that fails, take the first one.
    """
    if len(l) == 1:
        return l[0]
    else:
        inc = True
        toffset = 0
        defOffset = 2

        while abs(toffset) <= max(times):
            
            checkTime = time+toffset

            if checkTime in phenotypesAt:

                for a in l:
                    if a in phenotypesAt[checkTime]:
                        #print l, time, a
                        return a

            if inc:
                inc = False
                toffset *= -1
                toffset += defOffset
            else:
                inc = True
                toffset *= -1

        # here we came only if there is nothing at right time
        return l[0]
        #print l, time

def getGroup2(l, time):
    """
    Return phenotype from list of phenotypes. If there are more phenotypes, 
    return most advanced one.
    """

    if len(l) == 1:
        return l[0]

    else:
        try:
            sez = []
            for a in l:
                for i,p in enumerate(phentime):
                    try:
                        ind = p.index(a)
                        #print "appending", l, a, i, ind
                        sez.append((i,ind))
                    except:
                        pass

            sez = sorted(sez, reverse=True)
            
            sez1 = [ a for a in sez if a[0] == sez[0][0] ]
            sez1 = sorted(sez1)

            nt = phentime[sez1[0][0]][sez1[0][1]]
            #print l, sez, sez1, nt
            return nt
        except:
            return l[0]

def getGroup2J(l, ctime):

    from mMisc import flatten

    deambig = getGroup2(l, ctime) # get most advance phenotype
    ats = flatten( [ phenotypesAt[t] for t in phenotypesAt.keys() if t == ctime ] )
    befores = flatten([ phenotypesAt[t] for t in phenotypesAt.keys() if t < ctime ])
    afters = flatten([ phenotypesAt[t] for t in phenotypesAt.keys() if t > ctime ])
    if deambig in ats:
        return "on-time"
    elif deambig in befores:
        return "delayed"
    elif deambig in afters:
        return "advanced"
    else:
        fdsaf= fdsfdsdf
    

if __name__=="__main__":

    getGroup = getGroup2

    mutants,ptypes,dic = loadPhenotypes("phenotypes.txt")

    #take only first 4 phenotypes types
    ptypes = ptypes[:4]
    print "Using phenotypes", ptypes

    data = orange.ExampleTable("abc_v2.1.tab")

    #for testing
    #data = orange.ExampleTable(orange.Domain(data.domain.attributes[:5], data.domain.classVar), data)

    def getMutAndTime(ex):
        mut = str(ex[data.domain.classVar]).lower()
        mut = mut.replace("abc","").replace("-","")

        mutn = mut[:mut.index("+")]
        mutt = mut[(mut.index("+")+1):]

        return mutn, str(int(mutt))

    mutAndTimes = [ getMutAndTime(ex) for ex in data ]

    id = orange.newmetaid()
    data.domain.addmeta(id, orange.EnumVariable("mutant",values=sorted(set(nth(mutAndTimes,0)))))

    id = orange.newmetaid()
    data.domain.addmeta(id, orange.EnumVariable("time",values=sorted(set(nth(mutAndTimes,1)))))

    for ptype in ptypes:

        ctime = int(ptype.replace("Hrs",""))

        values = [ dic[(mut,ptype)] for mut in mutants ]
        values = [ getGroup(v.split(','), ctime) for v in values ]

        toVal = dict(zip(mutants,values))

        id = orange.newmetaid()
        data.domain.addmeta(id, orange.EnumVariable(ptype,values=sorted(set(values))))

        for i,ex in enumerate(data):
            mutn, mutt = getMutAndTime(ex)

            data[i]["mutant"] = mutn
            data[i]["time"] = mutt

            data[i][ptype] = toVal.get(mutn, "?")

    data.save("abc_v2.1_phen.tab")
