import orange, os, tempfile

def keepOnlyDiscreteAttr(data):
    """
    Walks through a domain and removes any non-discrete attribute.
    """

    attrs = []
    for a in data.domain.attributes:
        if a.varType == orange.VarTypes.Discrete:
            attrs.append(a)

    ndom = orange.Domain(attrs, data.domain.classVar)
    ndata = orange.ExampleTable(ndom, data)
    return ndata


def takeOnlyExamples(data, maxSize):
    """
    Takes only maxSize examples
    BUG: RANDOM 
    """
    size2 = min(maxSize, len(data))
    if size2 != len(data):
        indices2 = orange.MakeRandomIndices2(p0=size2)
        indices2.stratified = indices2.Stratified
        ind = indices2(data)
        datac = data.select(ind, 0)
        data = datac
    return data


def joinCV(data, classValues=None, joiner="+"):
    """
    Takes only specified class values. Join
    them accorting to the group
    """

    cv = data.domain.classVar

    oldcvals = [ a for a in cv.values ]

    #if not classValues:
    #    classValues = [ oldcvals[0], oldcvals[1] ]

    toJoin = []

    for vals in classValues:
        if isinstance(vals, list) or isinstance(vals, tuple):
            toJoin.append(list(vals))
        else:
            toJoin.append([vals])

    classValues = reduce(lambda x,y: x+y, toJoin)
    classValues = [ str(a) for a in classValues ] # ok class values

    if joiner in "".join(classValues):
        raise Exception("Joiner '%s' already in class values," % joiner)

    #dictionary of old class -> new class
    mapval = {}
    nclassvalues = [] # need to preserver order

    for joinvals in toJoin:
        joinvalsn = joiner.join([ str(val) for val in sorted(joinvals) ])
        nclassvalues.append(joinvalsn)

        for val in joinvals:
            mapval[str(val)] = joinvalsn

    #take only examples with classValues classes
    nclass = orange.EnumVariable(cv.name, values=nclassvalues)
    ndom = orange.Domain(data.domain.attributes, nclass)

    examples = []
    for ex in data:
        if ex[cv] in classValues:
            vals = [ ex[a] for a in data.domain.attributes ]
            vals.append(mapval[str(ex[cv].value)])
            examples.append(vals)

    data = orange.ExampleTable(ndom, examples)

    return data

def bootstrap(examples, rand):
    """
    Draw a bootstrap sample.
    """

    selection = []
    for j in range(len(examples)):
        selection.append(rand.randrange(len(examples)))
    data = examples.getitems(selection)
    return data

def getAttributes(data, lista):
    """
    Get only specified attributes from the dataset.
    """

    attrs = []
    for a in lista:
        attrs.append(data.domain.attributes[a])

    ndom = orange.Domain(attrs, data.domain.classVar)
    ndata = orange.ExampleTable(ndom, data)
    return ndata

def dropExamplesUC(data):
    """
    Drops examples with unknown class.
    """
    pp = orange.Preprocessor_drop()
    pp.values[data.domain.classVar] = "?"
    data2 = pp(data)
    return data2


def exampleTableToString(data):
    """
    Saves example table to string in tab format.
    """
    fname = tempSuffix("nekaj.tab")
    data.save(fname)
    f = open(fname, "rt")
    s = f.read()
    f.close()
    os.remove(fname)    
    return s

def exampleTableFromString(s):
    """
    Loads example table from string format.
    """
    fname = tempSuffix("nekaj.tab")
    f = open(fname, "wt")
    f.write(s)
    f.close()

    datao = orange.ExampleTable(fname)
    os.remove(fname)    
    return datao
 
    
def replaceAttrDesc(domain, rlist):
    """
    Replace attributes according to list of pair with
    contents of [ (attrToReplace, attrToReplaceItWith) ].
    Returns new domain.
    """

    repmap = dict([(domain[a],b) for a,b in rlist])
    alist = []

    for at in domain:
        #replace attribute with replacement or itself
        alist.append(repmap.get(at,at))
    
    #change class correctly
    return orange.Domain(alist, domain.classVar != None)


def shuffleAttribute(data, attribute, locations):
    """
    Destructive!
    """
    attribute = data.domain[attribute]
    l = [None]*len(data)
    for i in range(len(data)):
        l[locations[i]] = data[i][attribute]
    for i in range(len(data)):
        data[i][attribute] = l[i]

def shuffleAllAttributes(data, rand):
    """
    Randomly shuffle all attributes.
    Destructive!
    """
    for at in data.domain.attributes:
        locations = range(len(data))
        rand.shuffle(locations)
        shuffleAttribute(data, at, locations)

def tempSuffix(orig):
    """
    Returns name of created temporary file. 
    Preserves suffix.
    """
    try:
        name = orig[orig.rindex("."):]
        a,b = tempfile.mkstemp(name)
    except:
        a,b = tempfile.mkstemp()
        
    os.close(a)
    return b



